package com.jeffhanson.protocol;

import com.jeffhanson.resources.Resource;
import com.jeffhanson.services.Service;
import com.jeffhanson.services.ServiceCache;
import com.jeffhanson.services.ServiceException;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import java.io.IOException;

/**
 * User: jhanson
 * Date: Jun 10, 2008
 * Time: 8:08:06 PM
 */
public class HttpAdapter
  extends HttpServlet
{
  private void returnResourceResponse(HttpServletResponse response, Resource resource)
    throws IOException
  {
    response.setContentType(resource.getMimeType());
    response.setContentLength(resource.getContentLength());

    if (resource.canBeStringified())
    {
      response.getWriter().write(resource.toString());
    }
    else
    {
      ServletOutputStream outStream = response.getOutputStream();
      outStream.write(resource.getBytes());
      outStream.flush();
    }
  }

  protected void doPut(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    Service service = null;
    Resource resource = null;

    try
    {
      service = ServiceCache.get(request.getPathInfo());
      service.updateResource(request.getPathInfo(), getServletContext(), request);
      resource = service.readResource(request.getPathInfo(), getServletContext());
    }
    catch (ServiceException e)
    {
      e.printStackTrace();
      throw new ServletException(e.getMessage());
    }

    returnResourceResponse(response, resource);
  }

  protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    Service service = null;
    Resource resource = null;
    
    try
    {
      service = ServiceCache.get(request.getPathInfo());
      resource = service.readResource(request.getPathInfo(), getServletContext());
    }
    catch (ServiceException e)
    {
      e.printStackTrace();
      throw new ServletException(e.getMessage());
    }

    returnResourceResponse(response, resource);
  }

  protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    Service service = null;
    Resource resource = null;

    try
    {
      service = ServiceCache.get(request.getPathInfo());
      service.createResource(request.getPathInfo(), getServletContext(), request);
      resource = service.readResource(request.getPathInfo(), getServletContext());
    }
    catch (ServiceException e)
    {
      e.printStackTrace();
      throw new ServletException(e.getMessage());
    }

    returnResourceResponse(response, resource);
  }

  protected void doDelete(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    Service service = null;

    try
    {
      service = ServiceCache.get(request.getPathInfo());
      service.deleteResource(request.getPathInfo(), getServletContext());
    }
    catch (ServiceException e)
    {
      e.printStackTrace();
      throw new ServletException(e.getMessage());
    }

    response.sendError(HttpServletResponse.SC_NOT_FOUND, "Service successfully deleted");
  }

  protected long getLastModified(HttpServletRequest request)
  {
    Service service = null;
    long lastModified = 0;

    try
    {
      service = ServiceCache.get(request.getPathInfo());
      lastModified = service.getResourceLastModified(request.getPathInfo(), getServletContext());
    }
    catch (ServiceException e)
    {
      e.printStackTrace();
    }

    return lastModified;
  }
}
