package com.jeffhanson.services;

import com.jeffhanson.resources.Resource;
import com.jeffhanson.resources.ResourceAdapter;
import com.jeffhanson.resources.ResourceException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.ServletContext;

/**
 * User: jhanson
 * Date: Jun 10, 2008
 * Time: 8:16:50 PM
 */
public class ServiceCache
{
  /* todo - add logic to configure instance from dynamic data */
  private static ServiceCache instance = new ServiceCache();

  // =====================================================
  // non-public static methods
  // =====================================================

  private static ServiceCache getInstance()
  {
    return instance;
  }

  // =====================================================
  // public static methods
  // =====================================================

  public static Service get(String servicePath)
    throws ServiceException
  {
    return getInstance().getService(servicePath);
  }

  // =====================================================
  // non-public instance methods
  // =====================================================

  private Service getService(String servicePath)
    throws ServiceException
  {
    if (servicePath == null || servicePath.length() == 0)
    {
      throw new ServiceException("Service path is empty in "
                                 + getClass().getName()
                                 + ".readService()");
    }

    // create a simple service to return
    //
    return new Service()
    {
      public Resource readResource(String resourcePath, ServletContext servletContext)
        throws ServiceException
      {
        try
        {
          return ResourceAdapter.getInstance().readResource(resourcePath, servletContext);
        }
        catch (ResourceException e)
        {
          throw new ServiceException(e);
        }
      }

      public void createResource(String resourcePath, ServletContext servletContext, HttpServletRequest request)
        throws ServiceException
      {
        try
        {
          ResourceAdapter.getInstance().createResource(resourcePath, servletContext, request);
        }
        catch (ResourceException e)
        {
          throw new ServiceException(e);
        }
      }

      public void updateResource(String resourcePath, ServletContext servletContext, HttpServletRequest request)
        throws ServiceException
      {
        try
        {
          ResourceAdapter.getInstance().updateResource(resourcePath, servletContext, request);
        }
        catch (ResourceException e)
        {
          throw new ServiceException(e);
        }
      }

      public void deleteResource(String resourcePath, ServletContext servletContext)
        throws ServiceException
      {
        try
        {
          ResourceAdapter.getInstance().deleteResource(resourcePath, servletContext);
        }
        catch (ResourceException e)
        {
          throw new ServiceException(e);
        }
      }

      public long getResourceLastModified(String resourcePath, ServletContext servletContext)
        throws ServiceException
      {
        try
        {
          return ResourceAdapter.getInstance().getResourceLastModified(resourcePath, servletContext);
        }
        catch (ResourceException e)
        {
          throw new ServiceException(e);
        }
      }
    };
  }
}
