package com.jeffhanson.kernel.osgi;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleException;
import com.jeffhanson.kernel.KernelException;

/**
 * User: jhanson
 * Date: Jul 13, 2008
 * Time: 11:59:47 AM
 */
public class KernelActivator
  implements BundleActivator
{
  private BundleContext m_context = null;

  /**
   * Starts this activator
   * 
   * @param context - the bundle context for this activator
   */
  public void start(BundleContext context)
  {
    m_context = context;
  }

  /**
   * Stops this activator
   *
   * @param context - the bundle context for this activator
   */
  public void stop(BundleContext context)
  {
    m_context = null;
  }

  /**
   * Retrieves all bundles installed in this activator
   *
   * @return an array of all bundles installed in this activator
   */
  public Bundle[] getBundles()
  {
    if (m_context != null)
    {
      return m_context.getBundles();
    }

    return null;
  }

  /**
   * Retrieves the bundle context of this activator
   *
   * @return the bundle context
   */
  public BundleContext getContext()
  {
    return m_context;
  }

  /**
   * Installs a bundle in this activator
   *
   * @param bundleLocation
   * @return the newly installed bundle
   * @throws KernelException
   */
  public Object installBundle(String bundleLocation)
    throws KernelException
  {
    try
    {
      Bundle b = m_context.installBundle(bundleLocation, null);
      if (b != null)
      {
        b.start();
        return b;
      }
      throw new KernelException("Unable to install bundle: " + bundleLocation);
    }
    catch (Exception ex)
    {
      throw new KernelException("Bundle installation error: " + ex);
    }
  }

  /**
   * Uninstalls a bundle from this activator
   *
   * @param bundleLocation - the location of the bundle
   * @throws KernelException
   */
  public void uninstallBundle(String bundleLocation)
    throws KernelException
  {
    if (m_context == null)
    {
      throw new KernelException(getClass().getName()
                                + ".uninstallBundle() context is null");
    }

    Bundle[] bundles = m_context.getBundles();
    if (bundles == null || bundles.length <= 0)
    {
      throw new KernelException(getClass().getName()
                                + ".uninstallBundle() bundle list empty");
    }

    for (int i = 0; i < bundles.length; i++)
    {
      if (bundles[i].getLocation().equalsIgnoreCase(bundleLocation))
      {
        try
        {
          bundles[i].uninstall();
          return;
        }
        catch (BundleException e)
        {
          throw new KernelException(getClass().getName()
                                    + ".uninstallBundle() error: " + e);
        }
      }
    }

    throw new KernelException(getClass().getName()
                              + ".uninstallBundle() bundle not found: "
                              + bundleLocation);
  }
}
