package com.jeffhanson.mediation.logging;

import com.jeffhanson.mediation.Message;

/**
 * User: jhanson
 * Date: Aug 1, 2008
 * Time: 8:34:52 PM
 */
public class LoggingModuleFactory
{
  public static LoggingModule getLoggingModule(Message message)
    throws LoggingModuleFactoryException
  {
    String loggingModuleClsName = System.getProperty("com.jeffhanson.logging.module");
    if (null == loggingModuleClsName || loggingModuleClsName.length() <= 0)
    {
      throw new LoggingModuleFactoryException("LoggingModule system property is not set");
    }

    try
    {
      Class cls = Class.forName(loggingModuleClsName);
      if (com.jeffhanson.mediation.logging.LoggingModule.class.isAssignableFrom(cls))
      {
        Object obj = cls.newInstance();
        LoggingModule loggingModule = (LoggingModule)obj;
        loggingModule.initialize();
        return loggingModule;
      }
      else
      {
        throw new LoggingModuleFactoryException("Logging module system property is not derived from the LoggingModule class");
      }
    }
    catch (Exception e)
    {
      throw new LoggingModuleFactoryException(e);
    }
  }
}
