package com.jeffhanson.mediation.transform;

import com.jeffhanson.mediation.Message;
import org.jrdf.SortedMemoryJRDFFactory;
import org.jrdf.writer.rdfxml.MemRdfXmlWriter;
import org.jrdf.util.ClosableIterator;
import org.jrdf.util.EscapeURL;
import org.jrdf.parser.rdfxml.GraphRdfXmlParser;
import org.jrdf.parser.Parser;
import org.jrdf.graph.Graph;
import org.jrdf.graph.Triple;
import org.jrdf.graph.TripleImpl;

import java.net.URL;
import java.io.InputStream;
import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

/**
 * User: jhanson
 * Date: Aug 1, 2008
 * Time: 9:28:20 PM
 */
public class RDFTransformModule
  implements TransformModule
{
  public void initialize()
  {
  }

  public void processInboundMessage(Message message)
    throws TransformModuleException
  {
    if (message.getContentType().equalsIgnoreCase("application/rdf+xml") == false)
    {
      throw new TransformModuleException("Invalid content type encountered: " + message.getContentType());
    }

    String content = message.getContent();

    InputStream inStream = null;

    try
    {
      URL url = new URL("http://example.org/enterprise/mashups");
      inStream = new ByteArrayInputStream(content.getBytes());
      final Graph rdfMemGraph = SortedMemoryJRDFFactory.getFactory().getNewGraph();
      Parser parser = new GraphRdfXmlParser(rdfMemGraph);
      parser.parse(inStream, EscapeURL.toEscapedString(url));

      ClosableIterator<Triple> iter = null;

      try
      {
        iter = rdfMemGraph.find(org.jrdf.graph.AnySubjectNode.ANY_SUBJECT_NODE,
                                org.jrdf.graph.AnyPredicateNode.ANY_PREDICATE_NODE,
                                org.jrdf.graph.AnyObjectNode.ANY_OBJECT_NODE);
        while (iter.hasNext())
        {
          Triple triple = iter.next();
          if (triple.getObject().toString().contains("@example.org"))
          {
            String objectStr = triple.getObject().toString();
            objectStr = objectStr.replace("@example.org", "@jeffhanson.com");
            if (objectStr.startsWith("\""))
            {
              objectStr = objectStr.substring(1);
            }
            if (objectStr.endsWith("\""))
            {
              objectStr = objectStr.substring(0, objectStr.length() - 1);
            }

            rdfMemGraph.add(triple.getSubject(),
                            triple.getPredicate(),
                            new org.jrdf.graph.local.LiteralImpl(objectStr));
            rdfMemGraph.remove(triple);
          }
        }

        iter.close();
        iter = null;

        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        MemRdfXmlWriter writer = new MemRdfXmlWriter();
        writer.write(rdfMemGraph, outStream);

        message.setContent(outStream.toString());
      }
      finally
      {
        if (null != iter)
        {
          iter.close();
        }
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
    finally
    {
      if (null != inStream)
      {
        try
        {
          inStream.close();
        }
        catch (IOException e)
        {
          e.printStackTrace();
        }
      }
    }
  }
}
