package com.jeffhanson.mediation.transform;

import com.jeffhanson.mediation.Message;

/**
 * User: jhanson
 * Date: Aug 1, 2008
 * Time: 8:43:18 PM
 */
public class TransformModuleFactory
{
  public static TransformModule getTransformModule(Message message)
    throws TransformModuleFactoryException
  {
    try
    {
      Class cls = classForMessageType(message);
      if (com.jeffhanson.mediation.transform.TransformModule.class.isAssignableFrom(cls))
      {
        Object obj = cls.newInstance();
        TransformModule transformModule = (TransformModule)obj;
        transformModule.initialize();
        return transformModule;
      }
      else
      {
        throw new TransformModuleFactoryException("Transform module class is not derived from the TransformModule class");
      }
    }
    catch (Exception e)
    {
      throw new TransformModuleFactoryException(e);
    }
  }

  private static Class classForMessageType(Message message)
    throws TransformModuleFactoryException
  {
    String contentType = message.getContentType();

    if (contentType.equalsIgnoreCase("application/rdf+xml"))
    {
      return RDFTransformModule.class;
    }

    throw new TransformModuleFactoryException("Unknown message type encountered in TransformModuleFactory");
  }
}
