package org.mashups4j.governance;

import org.mashups4j.mediation.Message;
import org.mashups4j.governance.policies.PolicyModule;
import org.mashups4j.governance.policies.PolicyModuleFactory;
import org.mashups4j.governance.privacy.PrivacyModule;
import org.mashups4j.governance.privacy.PrivacyModuleFactory;
import org.mashups4j.governance.security.SecurityModule;
import org.mashups4j.governance.security.SecurityModuleFactory;

/**
 * User: jhanson
 * Date: Aug 9, 2008
 * Time: 4:48:23 PM
 */
public class SimpleGovernanceEngine
  implements GovernanceEngine
{
  private PolicyModule policyModule = null;
  private PrivacyModule privacyModule = null;
  private SecurityModule securityModule = null;

  public SimpleGovernanceEngine()
  {
  }

  public SimpleGovernanceEngine(PolicyModule policyModule,
                                PrivacyModule privacyModule,
                                SecurityModule securityModule)
  {
    this.policyModule = policyModule;
    this.privacyModule = privacyModule;
    this.securityModule = securityModule;
  }

  public void initialize()
  {
  }

  public void setPolicyModule(PolicyModule policyModule)
  {
    this.policyModule = policyModule;
  }

  public void setPrivacyModule(PrivacyModule privacyModule)
  {
    this.privacyModule = privacyModule;
  }

  public void setSecurityModule(SecurityModule securityModule)
  {
    this.securityModule = securityModule;
  }

  public void processInboundMessage(Message message)
    throws GovernanceException
  {
    try
    {
      if (null == policyModule)
      {
        policyModule = PolicyModuleFactory.getPolicyModule(message);
      }

      if (null == privacyModule)
      {
        privacyModule = PrivacyModuleFactory.getPrivacyModule(message);
      }

      if (null == securityModule)
      {
        securityModule = SecurityModuleFactory.getSecurityModule(message);
      }

      policyModule.processInboundMessage(message);
      privacyModule.processInboundMessage(message);
      securityModule.processInboundMessage(message);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      throw new GovernanceException(e);
    }
  }
}
