package org.mashups4j.governance.security;

import org.mashups4j.mediation.Message;

/**
 * User: jhanson
 * Date: Aug 1, 2008
 * Time: 8:35:16 PM
 */
public class SecurityModuleFactory
{
  public static SecurityModule getSecurityModule(Message message)
    throws SecurityModuleFactoryException
  {
    try
    {
      Class cls = classForMessageType(message);
      if (org.mashups4j.governance.security.SecurityModule.class.isAssignableFrom(cls))
      {
        Object obj = cls.newInstance();
        SecurityModule securityModule = (SecurityModule)obj;
        securityModule.initialize();
        return securityModule;
      }
      else
      {
        throw new SecurityModuleFactoryException("Security module class is not derived from the SecurityModule class");
      }
    }
    catch (Exception e)
    {
      throw new SecurityModuleFactoryException(e);
    }
  }

  private static Class classForMessageType(Message message)
    throws SecurityModuleFactoryException
  {
    String contentType = message.getContentType();

    if (contentType.equalsIgnoreCase("application/rdf+xml"))
    {
      return RDFSecurityModule.class;
    }

    throw new SecurityModuleFactoryException("Unknown message type encountered in SecurityModuleFactory");
  }
}
