package org.mashups4j.kernel;

import java.io.File;
import java.util.Properties;

/**
 * User: jhanson
 * Date: Jul 15, 2008
 * Time: 2:32:52 PM
 */
public class Daemon extends Thread
{
  private boolean running = false;
  private Kernel kernel = null;

  public static void main(String[] args)
    throws InterruptedException
  {
    try
    {
      Daemon daemon = new Daemon();
      daemon.initialize();
      daemon.startDaemon();

      while (true)
      {
        Thread.yield();
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  public void run()
  {
    running = true;

    System.out.println("Kernel daemon running...");

    long startTime = System.currentTimeMillis();
    long lastTime = startTime;

    while (running)
    {
      long currTime = System.currentTimeMillis();
      if ((currTime - lastTime) > 10000)
      {
        lastTime = currTime;
        listServices();
      }

      try
      {
        Thread.sleep(3000);
      }
      catch (InterruptedException e)
      {
        // ignore
      }
    }

    System.out.println("Kernel daemon stopped");
  }

  private synchronized void stopKernel()
  {
    if (null != kernel)
    {
      try
      {
        System.out.println("Stopping Kernel...");
        kernel.stop();
        kernel = null;
      }
      catch (KernelException e)
      {
        e.printStackTrace();
      }
    }
  }

  /**
   *
   * @return the kernel instance
   */
  public Kernel getKernel()
  {
    return kernel;
  }

  /**
   * Initializes this daemon
   *
   * @throws KernelException - on error
   */
  public void initialize()
    throws KernelException
  {
    System.setProperty("org.mashups4j.kernel", "org.mashups4j.kernel.osgi.OSGiKernel");
    String userDirPath = System.getProperty("user.dir");
    File servicesLocationDir = new File(userDirPath, "services");

    System.out.println("Creating Kernel with service dir at [" + servicesLocationDir + "]...");
    kernel = KernelFactory.getKernel(servicesLocationDir);
  }

  /**
   * Starts execution of this daemon
   *
   * @throws KernelException - on error
   */
  public void startDaemon()
    throws KernelException
  {
    Runtime.getRuntime().addShutdownHook(new Thread()
    {
      public void run()
      {
        System.out.println("Kernel daemon shutdown hook called");
        stopKernel();
      }
    });

    System.out.println("Starting Kernel...");
    kernel.start();
    
    //Thread daemon = new Daemon();

    /*
       Marking a "Thread" as a daemon thread means that it should
       automatically be killed when no other (non-daemon) application
       thread remains to be executed.
    */
    this.setDaemon(true);
    this.start();
  }

  /**
   * Stops execution of this daemon
   * 
   * @throws KernelException - on error
   */
  public void stopDaemon()
    throws KernelException
  {
    System.out.println("Kernel daemon stopping...");
    running = false;
    stopKernel();
  }

  public void listServices()
  {
    String[] serviceNames = getKernel().getInstalledServiceNames();
    if (null != serviceNames && serviceNames.length > 0)
    {
      for (int i = 0; i < serviceNames.length; i++)
      {
        String serviceName = serviceNames[i];
        System.out.println("Found installed service: " + serviceName);
      }
    }
  }

  public Object invokeService(String serviceName, Properties props)
    throws KernelException
  {
    Object service = getKernel().getServiceByName(serviceName);
    if (null == service)
    {
      throw new KernelException("Service [" + serviceName + "] not found.");
    }
    return kernel.invokeService(service, props);
  }
}
