package org.mashups4j.kernel;

import java.io.File;
import java.util.HashMap;

/**
 * User: jhanson
 * Date: Jul 14, 2008
 * Time: 11:52:25 PM
 */
public class KernelFactory
{
  private static HashMap<String, Kernel> kernels = new HashMap<String, Kernel>();

  public static Kernel getKernel(File servicesLocationDir)
    throws KernelException
  {
    Kernel kernel = kernels.get(servicesLocationDir.getAbsolutePath());
    if (null != kernel)
    {
      return kernel;
    }

    String kernelClsName = System.getProperty("org.mashups4j.kernel");
    if (null == kernelClsName || kernelClsName.length() <= 0)
    {
      throw new KernelException("Kernel system property is not set");
    }
    
    try
    {
      Class cls = Class.forName(kernelClsName);
      if (org.mashups4j.kernel.Kernel.class.isAssignableFrom(cls))
      {
        Object obj = cls.newInstance();
        kernel = (Kernel)obj;
        kernel.setBundlesLocation(servicesLocationDir);
        kernel.initialize();

        kernels.put(servicesLocationDir.getAbsolutePath(), kernel);

        return kernel;
      }
      else
      {
        throw new KernelException("Kernel system property is not derived from the Kernel class");
      }
    }
    catch (Exception e)
    {
      throw new KernelException(e);
    }
  }
}
