package org.mashups4j.mediation;

import org.mashups4j.mediation.auditing.AuditModule;
import org.mashups4j.mediation.auditing.AuditModuleFactory;
import org.mashups4j.mediation.logging.LoggingModule;
import org.mashups4j.mediation.logging.LoggingModuleFactory;
import org.mashups4j.governance.GovernanceEngine;
import org.mashups4j.governance.GovernanceEngineFactory;
import org.mashups4j.mediation.transform.TransformModule;
import org.mashups4j.mediation.transform.TransformModuleFactory;

/**
 * User: jhanson
 * Date: Aug 1, 2008
 * Time: 9:50:15 PM
 */
public class SimpleMediator
  implements Mediator
{
  private AuditModule auditModule = null;
  private LoggingModule loggingModule = null;
  private GovernanceEngine governanceEngine = null;
  private TransformModule transformModule = null;

  public SimpleMediator()
  {
  }

  public SimpleMediator(AuditModule auditModule,
                        LoggingModule loggingModule,
                        GovernanceEngine governanceEngine,
                        TransformModule transformModule)
  {
    this.auditModule = auditModule;
    this.loggingModule = loggingModule;
    this.governanceEngine = governanceEngine;
    this.transformModule = transformModule;
  }

  public void initialize()
  {
  }

  public void setAuditModule(AuditModule auditModule)
  {
    this.auditModule = auditModule;
  }

  public void setLoggingModule(LoggingModule loggingModule)
  {
    this.loggingModule = loggingModule;
  }

  public void setGovernanceEngine(GovernanceEngine governanceEngine)
  {
    this.governanceEngine = governanceEngine;
  }

  public void setTransformModule(TransformModule transformModule)
  {
    this.transformModule = transformModule;
  }

  public void mediate(Message message)
    throws MediationException
  {
    try
    {
      if (null == auditModule)
      {
        auditModule = AuditModuleFactory.getAuditModule(message);
      }

      if (null == loggingModule)
      {
        loggingModule = LoggingModuleFactory.getLoggingModule(message);
      }

      if (null == governanceEngine)
      {
        governanceEngine = GovernanceEngineFactory.newInstance();
      }

      if (null == transformModule)
      {
        transformModule = TransformModuleFactory.getTransformModule(message);
      }

      auditModule.processInboundMessage(message);
      loggingModule.processInboundMessage(message);
      governanceEngine.processInboundMessage(message);
      transformModule.processInboundMessage(message);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      throw new MediationException(e);
    }
  }
}
