package org.mashups4j.monitoring;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;

/**
 * User: jhanson
 * Date: Aug 3, 2008
 * Time: 7:48:44 PM
 */
public class SimpleEventManager
  implements EventManager,
             ProtocolAdapterListener,
             Runnable
{
  private ArrayList<EventProtocolAdapter> eventProtocolAdapters = new ArrayList<EventProtocolAdapter>();
  private ArrayList<EventManagerListener> listeners = new ArrayList<EventManagerListener>();
  private Thread workerThread = null;
  private boolean running = false;

  public SimpleEventManager()
  {
    EventProtocolAdapter protocolAdapter = new LocalProtocolAdapter();
    eventProtocolAdapters.add(protocolAdapter);
    protocolAdapter.addEventReceivedListener(this);
  }

  public SimpleEventManager(EventProtocolAdapter[] protocolAdapters)
  {
    eventProtocolAdapters.addAll(Arrays.asList(protocolAdapters));
  }

  public void run()
  {
    running = true;

    System.out.println("SimpleEventManager started");

    while (running)
    {
      Thread.yield();
    }

    System.out.println("SimpleEventManager stopped");

    running = false;
  }

  public void addProtocolAdapter(EventProtocolAdapter protocolAdapter)
  {
    eventProtocolAdapters.add(protocolAdapter);
    if (running)
    {
      try
      {
        protocolAdapter.start();
      }
      catch (EventProtocolAdapterException e)
      {
        e.printStackTrace();
      }
    }
  }

  public void removeProtocolAdapter(EventProtocolAdapter protocolAdapter)
  {
    int idx = eventProtocolAdapters.indexOf(protocolAdapter);
    if (idx >= 0)
    {
      try
      {
        protocolAdapter.stop();
      }
      catch (EventProtocolAdapterException e)
      {
        e.printStackTrace();
      }
      eventProtocolAdapters.remove(protocolAdapter);
    }
  }

  public void initialize()
    throws EventManagerException
  {
  }

  public void addEventManagerListener(EventManagerListener listener)
  {
    listeners.add(listener);
  }

  public void start()
    throws EventManagerException
  {
    Iterator<EventProtocolAdapter> iter = eventProtocolAdapters.iterator();
    while (iter.hasNext())
    {
      EventProtocolAdapter eventProtocolAdapter = iter.next();
      try
      {
        eventProtocolAdapter.start();
      }
      catch (EventProtocolAdapterException e)
      {
        e.printStackTrace();
        throw new EventManagerException(e);
      }
    }

    workerThread = new Thread(this);
    workerThread.start();
  }

  public void stop()
    throws EventManagerException
  {
    Exception exception = null;

    Iterator<EventProtocolAdapter> iter = eventProtocolAdapters.iterator();
    while (iter.hasNext())
    {
      EventProtocolAdapter eventProtocolAdapter = iter.next();
      try
      {
        eventProtocolAdapter.stop();
      }
      catch (EventProtocolAdapterException e)
      {
        e.printStackTrace();
        exception = e;
      }
    }

    running = false;

    if (null != exception)
    {
      throw new EventManagerException(exception);
    }
  }

  public void eventOccurred(ProtocolAdapterEvent evt)
  {
    ManagementEvent mgmtEvt = new ManagementEvent(evt.getSource(),
                                                  evt.getTimeStamp(),
                                                  evt.getType(),
                                                  evt.getContent());

    Iterator<EventManagerListener> iter = listeners.iterator();
    while (iter.hasNext())
    {
      EventManagerListener listener = iter.next();
      listener.eventOccurred(mgmtEvt);
    }
  }
}
