package org.mashups4j.resources;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * User: jhanson
 * Date: Jun 11, 2008
 * Time: 4:29:19 PM
 */
public class ResourceAdapter
  extends HttpServlet
{
  private void serializeResource(HttpServletResponse response, Resource resource)
    throws IOException
  {
    response.setContentType(resource.getMimeType());
    response.setContentLength(resource.getContentLength());

    if (resource.canBeStringified())
    {
      response.getWriter().write(resource.toString());
    }
    else
    {
      ServletOutputStream outStream = response.getOutputStream();
      outStream.write(resource.getBytes());
      outStream.flush();
    }
  }

  protected void doGet(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException
  {
    try
    {
      System.out.println("Context Path: " + req.getContextPath());
      System.out.println("Path Info: " + req.getPathInfo());
      System.out.println("Path Translated: " + req.getPathTranslated());
      System.out.println("Request URI: " + req.getRequestURI());
      System.out.println("Servlet Path: " + req.getServletPath());
      
      Resource resource =
        FileSystemResourceCache.getInstance(true).readResource(req.getPathInfo(),
                                                               getServletContext(),
                                                               false);
      serializeResource(res, resource);
    }
    catch (ResourceException e)
    {
      e.printStackTrace();
      throw new ServletException(e);
    }
  }

  protected void doPost(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException
  {
    try
    {
      FileSystemResourceCache.getInstance(true).updateResource(req.getPathInfo(),
                                                 getServletContext(),
                                                 req);
      PrintWriter out = res.getWriter();
      out.println("success");
    }
    catch (ResourceException e)
    {
      e.printStackTrace();
      throw new ServletException(e);
    }
  }

  protected void doPut(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException
  {
    try
    {
      FileSystemResourceCache.getInstance(true).createResource(req.getPathInfo(),
                                                 getServletContext(),
                                                 req);
      PrintWriter out = res.getWriter();
      out.println("success");
    }
    catch (ResourceException e)
    {
      e.printStackTrace();
      throw new ServletException(e);
    }
  }

  protected void doDelete(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException
  {
    try
    {
      FileSystemResourceCache.getInstance(true).deleteResource(req.getPathInfo(),
                                                 getServletContext());
      PrintWriter out = res.getWriter();
      out.println("success");
    }
    catch (ResourceException e)
    {
      e.printStackTrace();
      throw new ServletException(e);
    }
  }

  protected long getLastModified(HttpServletRequest req)
  {
    try
    {
      return FileSystemResourceCache.getInstance(true).getResourceLastModified(req.getPathInfo(),
                                                                 getServletContext());
    }
    catch (ResourceException e)
    {
      e.printStackTrace();
    }

    return 0;
  }
}
