package org.mashups4j.services;

import org.mashups4j.web.Model;
import org.mashups4j.kernel.Kernel;
import org.mashups4j.kernel.KernelFactory;
import org.mashups4j.kernel.KernelException;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import java.io.IOException;
import java.io.File;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.util.Properties;
import java.util.Enumeration;

/**
 * User: jhanson
 * Date: Jun 10, 2008
 * Time: 8:08:06 PM
 */
public class HttpAdapter
  extends HttpServlet
{
  private Model successModel = new Model()
  {
    String content = "success";

    public String getMimeType()
    {
      return "text/plain";
    }

    public int getContentLength()
    {
      return toString().length();
    }

    public boolean canBeStringified()
    {
      return true;
    }

    public byte[] getBytes()
    {
      return toString().getBytes();
    }

    public void setContents(Object content)
    {
      this.content = content.toString();
    }

    public String toString()
    {
      return content;
    }
  };
  
  private static Kernel kernel = null;

  /**
   * @param response
   * @param model
   * @throws IOException
   */
  private void serializeModel(HttpServletResponse response, Model model)
    throws IOException
  {
    response.setContentType(model.getMimeType());
    response.setContentLength(model.getContentLength());

    if (model.canBeStringified())
    {
      response.getWriter().write(model.toString());
    }
    else
    {
      ServletOutputStream outStream = response.getOutputStream();
      outStream.write(model.getBytes());
      outStream.flush();
    }
  }

  public void init()
    throws ServletException
  {
    if (null == kernel)
    {
      System.setProperty("org.mashups4j.kernel", "org.mashups4j.kernel.osgi.OSGiKernel");
      String realPath = getServletContext().getRealPath("/");
      File servicesLocationDir = new File(realPath, "services");

      System.out.println("Creating Kernel with service dir at [" + servicesLocationDir + "]...");
      try
      {
        kernel = KernelFactory.getKernel(servicesLocationDir);
        kernel.start();
        
        // wait for kernel to discover and load services
        //
        long startTime = System.currentTimeMillis();
        long currTime = startTime;
        while ((currTime - startTime) < 5000)
        {
          Thread.yield();
          currTime = System.currentTimeMillis();
        }
      }
      catch (KernelException e)
      {
        e.printStackTrace();
        throw new ServletException(e);
      }
    }
  }

  public void destroy()
  {
    if (null != kernel)
    {
      try
      {
        kernel.stop();
        kernel = null;
      }
      catch (KernelException e)
      {
        e.printStackTrace();
      }
    }
  }

  /**
   * @param request
   * @param response
   * @throws ServletException
   * @throws IOException
   */
  protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    Object service = null;
    Model model = null;

    try
    {
      service = ServiceCache.get(getKernel(), request.getPathInfo());
      if (service instanceof Service)
      {
        model = ((Service)service).read(request.getPathInfo(), request.getParameterMap(), getServletContext());
      }
      else
      {
        model = ServiceHelper.dynamicallyInvokeService(service, request);
      }
    }
    catch (ServiceException e)
    {
      e.printStackTrace();
      throw new ServletException(e.getMessage());
    }

    serializeModel(response, model);
  }

  /**
   * @param request
   * @param response
   * @throws ServletException
   * @throws IOException
   */
  protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    try
    {
      Object service = ServiceCache.get(getKernel(), request.getPathInfo());
      if (service instanceof Service)
      {
        ((Service)service).create(request.getPathInfo(), request.getParameterMap(), getServletContext());
      }
    }
    catch (ServiceException e)
    {
      e.printStackTrace();
      throw new ServletException(e.getMessage());
    }

    serializeModel(response, successModel);
  }

  /**
   * @param request
   * @param response
   * @throws ServletException
   * @throws IOException
   */
  protected void doPut(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    try
    {
      Object service = ServiceCache.get(getKernel(), request.getPathInfo());
      if (service instanceof Service)
      {
        ((Service)service).update(request.getPathInfo(), request.getParameterMap(), getServletContext());
      }
    }
    catch (ServiceException e)
    {
      e.printStackTrace();
      throw new ServletException(e.getMessage());
    }

    serializeModel(response, successModel);
  }

  /**
   * @param request
   * @param response
   * @throws ServletException
   * @throws IOException
   */
  protected void doDelete(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    try
    {
      Object service = ServiceCache.get(getKernel(), request.getPathInfo());
      if (service instanceof Service)
      {
        ((Service)service).delete(request.getPathInfo(), getServletContext());
      }
    }
    catch (ServiceException e)
    {
      e.printStackTrace();
      throw new ServletException(e.getMessage());
    }

    serializeModel(response, successModel);
  }

  /**
   * @param request
   * @return last modified date
   */
  protected long getLastModified(HttpServletRequest request)
  {
    long lastModified = 0;

    try
    {
      Object service = ServiceCache.get(getKernel(), request.getPathInfo());
      if (service instanceof Service)
      {
        lastModified = ((Service)service).getLastModified(request.getPathInfo(), getServletContext());
      }
    }
    catch (ServiceException e)
    {
      e.printStackTrace();
    }

    return lastModified;
  }

  /**
   *
   * @return Kernel
   */
  public Kernel getKernel()
  {
    return kernel;
  }
}
