package org.mashups4j.timeseries;

import java.util.LinkedHashMap;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * User: jhanson
 * Date: Aug 24, 2008
 * Time: 7:23:33 PM
 */
public class TimeSeries
{
  private LinkedHashMap<TimePeriod, Double> intervals = new LinkedHashMap<TimePeriod, Double>();
  private String name = "";
  private ArrayList<TimeSeriesChangeListener> timeSeriesChangeListeners = new ArrayList<TimeSeriesChangeListener>();

  public TimeSeries(String name)
  {
    this.name = name;
  }

  public void addInterval(final TimePeriod period, final double value)
  {
    intervals.put(period, value);

    Interval interval = new Interval()
    {
      public TimePeriod getPeriod()
      {
        return period;
      }

      public double getValue()
      {
        return value;
      }
    };

    TimeSeriesChangedEvent event = new TimeSeriesChangedEvent(this, interval);

    Iterator<TimeSeriesChangeListener> iter = timeSeriesChangeListeners.iterator();
    while (iter.hasNext())
    {
      TimeSeriesChangeListener timeSeriesChangeListener = iter.next();
      timeSeriesChangeListener.intervalAdded(event);
    }
  }

  public int getIntervalCount()
  {
    return intervals.size();
  }

  public String getName()
  {
    return name;
  }

  public TimePeriod getKey(int i)
  {
    return (TimePeriod)intervals.keySet().toArray()[i];
  }

  public Double getValue(int i)
  {
    return intervals.get(i);
  }

  public void addTimeSeriesChangeListener(TimeSeriesChangeListener timeSeriesChangeListener)
  {
    this.timeSeriesChangeListeners.add(timeSeriesChangeListener);
  }

  public void removeTimeSeriesChangeListener(TimeSeriesChangeListener timeSeriesChangeListener)
  {
    this.timeSeriesChangeListeners.remove(timeSeriesChangeListener);
  }
}
