package org.mashups4j.web;

import org.apache.commons.digester.Digester;
import org.xml.sax.SAXException;

import javax.servlet.ServletContext;
import java.util.WeakHashMap;
import java.util.Iterator;
import java.util.HashMap;
import java.io.IOException;
import java.net.URL;

/**
 * User: jhanson
 * Date: Nov 8, 2008
 * Time: 12:42:45 PM
 */
public class ActionCache
{
  private static ActionCache instance = null;

  // =====================================================
  // non-public static methods
  // =====================================================

  private static synchronized ActionCache getInstance(ServletContext servletContext)
    throws IOException, SAXException
  {
    if (null == instance)
    {
      instance = new ActionCache(servletContext);
    }

    return instance;
  }

  // =====================================================
  // public static methods
  // =====================================================

  public static synchronized Action get(ServletContext servletContext, String actionPath)
    throws ActionException
  {
    Action action = null;
    
    try
    {
      action = getInstance(servletContext).getAction(actionPath);
    }
    catch (Exception e)
    {
      //e.printStackTrace();
      throw new ActionException(e);
    }

    return action;
  }

  // =====================================================
  // member fields
  // =====================================================

  private HashMap<String, Action> actionMap = new HashMap<String, Action>();

  // =====================================================
  // constructors
  // =====================================================

  public ActionCache(ServletContext servletContext)
    throws IOException, SAXException
  {
    Digester digester = new Digester();
    digester.setValidating(false);    
    
    // This method pushes the instance to the Digesters
    // object stack making its methods available to processing rules.
    //
    digester.push(this);

    // This set of rules calls the addAction method and passes
    // 2 parameters to the method.
    //
    digester.addCallMethod("actions/action", "addAction", 2);
    digester.addCallParam("actions/action/path", 0);
    digester.addCallParam("actions/action/classname", 1);

    // This method starts the parsing of the document.
    //

    URL actionsURL = null;
    if (null != servletContext)
    {
      actionsURL = servletContext.getResource("Actions.xml");
    }
    else
    {
      actionsURL = Thread.currentThread().getContextClassLoader().getResource("Actions.xml");
    }

    if (null == actionsURL)
    {
      throw new IOException("Actions.xml not found");
    }
    
    digester.parse(actionsURL);
  }

  // =====================================================
  // non-public instance methods
  // =====================================================

  private Action getAction(String actionPath)
    throws ActionException
  {
    if (actionPath == null || actionPath.length() == 0)
    {
      throw new ActionException("Action path is empty in "
                                 + getClass().getName()
                                 + ".getService()");
    }

    Action action = actionMap.get(actionPath);
    if (null == action)
    {
      throw new ActionException("Action [" + actionPath + "] not found in "
                                 + getClass().getName()
                                 + ".getAction()");
    }

    return action;
  }

  // =====================================================
  // public instance methods
  // =====================================================

  /**
   * called by Digester
   * 
   * @param path
   * @param classname
   */
  public void addAction(String path, String classname)
  {
    // create the Action and add to collection
    //
    Object actionObj = null;

    try
    {
      Class cls = Class.forName(classname);
      if (!Action.class.isAssignableFrom(cls))
      {
        throw new Exception("Action is not assignable from: " + classname);
      }
      
      actionObj = cls.newInstance();
    }
    catch (Exception e)
    {
      e.printStackTrace();
      return;
    }

    System.out.println("Adding [" + path + "] [" + actionObj + "] to actions cache");
    
    actionMap.put(path, (Action)actionObj);
  }
}
