package org.mashups4j.web;

import java.util.Iterator;
import java.util.ArrayList;

/**
 * User: jhanson
 * Date: Sep 22, 2008
 * Time: 11:08:48 PM
 */
public class ContactsValidator
  implements Validator
{
  public static final String CONTACT_NAME = "contact_name";
  public static final String CONTACT_ADDRESS = "contact_address";
  public static final String CONTACT_CITY = "contact_city";
  public static final String CONTACT_STATE = "contact_state";
  public static final String CONTACT_ZIP = "contact_zip";
  public static final String CONTACT_PHONE = "contact_phone";
  public static final String CONTACT_EMAIL = "contact_email";

  public static final String[] STATES = {
    "AL", "AK", "AZ", "AR", "CA", "CO", "CT", "DE", "DC", "FL",
    "GA", "HI", "ID", "IL", "IN", "IA", "KS", "KY", "LA", "ME",
    "MD", "MA", "MI", "MN", "MS", "MO", "MT", "NE", "NV", "NH",
    "NJ", "NM", "NY", "NC", "ND", "OH", "OK", "OR", "PA", "RI",
    "SC", "SD", "TN", "TX", "UT", "VT", "VA", "WA", "WV", "WI",
    "WY"
  };

  private static final ArrayList<String> fieldNames = new ArrayList<String>()
  {{
    add(CONTACT_NAME);
    add(CONTACT_ADDRESS);
    add(CONTACT_CITY);
    add(CONTACT_STATE);
    add(CONTACT_ZIP);
    add(CONTACT_PHONE);
    add(CONTACT_EMAIL);
  }};

  public Iterator<String> fieldNames()
  {
    return fieldNames.iterator();
  }

  public boolean validate(String fieldName, String fieldValue)
  {
    if (fieldName.equalsIgnoreCase(CONTACT_NAME))
    {
      return fieldValue.matches("^([a-z A-Z]+)$");
    }
    else if (fieldName.equalsIgnoreCase(CONTACT_ADDRESS))
    {
      return fieldValue.matches("^([a-z A-Z1-9]+)$");
    }
    else if (fieldName.equalsIgnoreCase(CONTACT_CITY))
    {
      return fieldValue.matches("^([a-z A-Z]+)$");
    }
    else if (fieldName.equalsIgnoreCase(CONTACT_STATE))
    {
      for (int i = 0; i < STATES.length; i++)
      {
        if (fieldValue.equalsIgnoreCase(STATES[i]))
        {
          return true;
        }
      }
    }
    else if (fieldName.equalsIgnoreCase(CONTACT_ZIP))
    {
      return fieldValue.matches("(^\\d{5}$)|(^\\d{5}-\\d{4}$)");
    }
    else if (fieldName.equalsIgnoreCase(CONTACT_PHONE))
    {
      return fieldValue.matches("^\\([1-9]\\d{2}\\)\\s?\\d{3}\\-\\d{4}$");
    }
    else if (fieldName.equalsIgnoreCase(CONTACT_EMAIL))
    {
      return fieldValue.matches("(^[a-z]([a-z_\\.]*)@([a-z_\\.]*)([.][a-z]{3})$)|(^[a-z]([a-z_\\.]*)@([a-z_\\.]*)(\\.[a-z]{3})(\\.[a-z]{2})*$)");
    }
    
    return false;
  }
}
