package org.mashups4j.web;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import java.io.IOException;
import java.util.Iterator;

/**
 * User: jhanson
 * Date: Sep 22, 2008
 * Time: 10:02:32 AM
 */
public class FrontController extends HttpServlet
{
  private String validateRequest(HttpServletRequest req)
  {
    StringBuffer outStrBuf = new StringBuffer();

    Validator validator = ValidatorFactory.getValidator(ValidatorFactory.CONTACTS_VALIDATOR);

    Iterator<String> iter = validator.fieldNames();
    while (iter.hasNext())
    {
      String fieldName = iter.next();
      String fieldValue = req.getParameter(fieldName);
      if ((null == fieldValue) || fieldValue.length() <= 0)
      {
        outStrBuf.append("<h3>" + fieldName + " not found. Press back button.</h3>\n");
      }
      else if (!validator.validate(fieldName, fieldValue))
      {
        outStrBuf.append("<h3>" + fieldName + " is invalid. Press back button.</h3>\n");
      }
    }

    return outStrBuf.toString();
  }

  private Model successModel = new Model()
  {
    String contents = "success";

    public String getMimeType()
    {
      return "text/plain";
    }

    public int getContentLength()
    {
      return toString().length();
    }

    public boolean canBeStringified()
    {
      return true;
    }

    public byte[] getBytes()
    {
      return toString().getBytes();
    }

    public void setContents(Object contents)
    {
      this.contents = contents.toString();
    }

    public String toString()
    {
      return contents;
    }
  };

  /**
   * @param response
   * @param model
   * @throws IOException
   */
  private void serializeModel(HttpServletResponse response, Model model)
    throws IOException
  {
    response.setContentType(model.getMimeType());
    response.setContentLength(model.getContentLength());

    if (model.canBeStringified())
    {
      response.getWriter().write(model.toString());
    }
    else
    {
      ServletOutputStream outStream = response.getOutputStream();
      outStream.write(model.getBytes());
      outStream.flush();
    }
  }

  /**
   * @param request
   * @param response
   * @throws ServletException
   * @throws IOException
   */
  protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    Model model = null;

    try
    {
      Action action = ActionCache.get(getServletContext(), request.getPathInfo());
      model = action.read(request.getPathInfo(), request.getParameterMap(), getServletContext());
    }
    catch (ActionException e)
    {
      e.printStackTrace();
      throw new ServletException(e.getMessage());
    }

    serializeModel(response, model);
  }

  /**
   * @param request
   * @param response
   * @throws ServletException
   * @throws IOException
   */
  protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    try
    {
      Action action = ActionCache.get(getServletContext(), request.getPathInfo());
      action.create(request.getPathInfo(), request.getParameterMap(), getServletContext());
    }
    catch (ActionException e)
    {
      e.printStackTrace();
      throw new ServletException(e.getMessage());
    }

    serializeModel(response, successModel);
  }

  /**
   * @param request
   * @param response
   * @throws ServletException
   * @throws IOException
   */
  protected void doPut(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    try
    {
      Action action = ActionCache.get(getServletContext(), request.getPathInfo());
      action.update(request.getPathInfo(), request.getParameterMap(), getServletContext());
    }
    catch (ActionException e)
    {
      e.printStackTrace();
      throw new ServletException(e.getMessage());
    }

    serializeModel(response, successModel);
  }

  /**
   * @param request
   * @param response
   * @throws ServletException
   * @throws IOException
   */
  protected void doDelete(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    try
    {
      Action action = ActionCache.get(getServletContext(), request.getPathInfo());
      action.delete(request.getPathInfo(), getServletContext());
    }
    catch (ActionException e)
    {
      e.printStackTrace();
      throw new ServletException(e.getMessage());
    }

    serializeModel(response, successModel);
  }
}
