package org.mashups4j.services;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;

import java.io.File;

import org.mashups4j.kernel.KernelFactory;
import org.mashups4j.kernel.KernelException;
import org.mashups4j.kernel.Kernel;

/**
 * User: jhanson
 * Date: Nov 8, 2008
 * Time: 5:16:03 PM
 */
public class ServiceCacheTest
  extends TestCase
{
  private static Kernel kernel = null;

  private static Kernel getKernel()
    throws Exception
  {
    if (null != kernel)
    {
      return kernel;
    }

    System.setProperty("org.mashups4j.kernel", "org.mashups4j.kernel.osgi.OSGiKernel");
    String userDirPath = System.getProperty("user.dir");
    File servicesLocationDir = new File(userDirPath, "services");

    System.out.println("Creating Kernel with service dir at [" + servicesLocationDir + "]...");

    try
    {
      kernel = KernelFactory.getKernel(servicesLocationDir);
      kernel.start();

      // wait for kernel to discover and load services
      //
      long startTime = System.currentTimeMillis();
      long currTime = startTime;
      while ((currTime - startTime) < 5000)
      {
        Thread.yield();
        currTime = System.currentTimeMillis();
      }
      
      return kernel;
    }
    catch (KernelException e)
    {
      e.printStackTrace();
      throw new Exception(e);
    }
  }

  public ServiceCacheTest(String name)
  {
    super(name);
  }

  public void setUp()
    throws Exception
  {
    super.setUp();
    getKernel();
  }

  public void tearDown()
    throws Exception
  {
    super.tearDown();
  }

  public void testGet()
    throws Exception
  {
    try
    {
      Object service1 = ServiceCache.get(getKernel(), "/twitter");
      assertNotNull(service1);

      Object service2 = ServiceCache.get(getKernel(), "/googlemaps");
      assertNotNull(service2);
    }
    catch (Throwable t)
    {
      t.printStackTrace();
      System.err.println(t.getMessage());
      throw new Exception(t);
    }

    try
    {
      Object bogusService = ServiceCache.get(getKernel(), "/bogus");
      System.out.println("<<< bogus service should have thrown an exception >>>");
      throw new Exception("bogus service should have thrown an exception");
    }
    catch (Throwable t)
    {
      //t.printStackTrace();
      // this should happen
    }
  }

  public static Test suite()
  {
    return new TestSuite(ServiceCacheTest.class);
  }
}
