package org.mashups4j.services;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.mashups4j.kernel.Kernel;
import org.mashups4j.kernel.KernelFactory;
import org.mashups4j.kernel.KernelException;
import org.mashups4j.web.Model;

import java.io.File;
import java.util.Properties;

/**
 * User: jhanson
 * Date: Nov 9, 2008
 * Time: 2:28:27 PM
 */
public class ServiceTest
  extends TestCase
{
  private static Kernel kernel = null;

  private static Kernel getKernel()
    throws Exception
  {
    if (null != kernel)
    {
      return kernel;
    }

    System.setProperty("org.mashups4j.kernel", "org.mashups4j.kernel.osgi.OSGiKernel");
    String userDirPath = System.getProperty("user.dir");
    File servicesLocationDir = new File(userDirPath, "services");

    System.out.println("Creating Kernel with service dir at [" + servicesLocationDir + "]...");

    try
    {
      kernel = KernelFactory.getKernel(servicesLocationDir);
      kernel.start();

      // wait for kernel to discover and load services
      //
      long startTime = System.currentTimeMillis();
      long currTime = startTime;
      while ((currTime - startTime) < 5000)
      {
        Thread.yield();
        currTime = System.currentTimeMillis();
      }

      return kernel;
    }
    catch (KernelException e)
    {
      e.printStackTrace();
      throw new Exception(e);
    }
  }

  public ServiceTest(String name)
  {
    super(name);
  }

  public void setUp()
    throws Exception
  {
    super.setUp();
  }

  public void tearDown()
    throws Exception
  {
    super.tearDown();
  }

  public void testExecute()
    throws Exception
  {
    Object service1 = ServiceCache.get(getKernel(), "/twitter");
    assertNotNull(service1);
    Properties props1 = new Properties();
    props1.put("targetUserID", "jhanson583");
    Model model1 = ServiceHelper.dynamicallyInvokeService(service1, props1);
    System.out.println("<<< Twitter service returned [" + model1.toString() + "] >>>");

    Object service2 = ServiceCache.get(getKernel(), "/googlemaps");
    assertNotNull(service2);
    Properties props2 = new Properties();
    props2.put("location", "Salt Lake City, UT");
    Model model2 = ServiceHelper.dynamicallyInvokeService(service2, props2);
    System.out.println("<<< GoogleMaps service returned [" + model2.toString() + "] >>>");

    Object service3 = ServiceCache.get(getKernel(), "/twoogle");
    assertNotNull(service3);
    Properties props3 = new Properties();
    props3.put("twitterUserID", "jhanson583");
    Model model3 = ServiceHelper.dynamicallyInvokeService(service3, props3);
    System.out.println("<<< Twoogle service returned [" + model3.toString() + "] >>>");
  }

  public static Test suite()
  {
    return new TestSuite(ServiceTest.class);
  }
}
