package org.mashups4j.services.google;

import org.mashups4j.services.utils.HTTPUtils;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.net.URLEncoder;
import java.io.StringReader;
import java.util.StringTokenizer;
import java.util.Properties;
import java.util.Iterator;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;

/**
 * User: jhanson
 * Date: Nov 6, 2008
 * Time: 9:24:05 PM
 *
 * Implementation of a Google Maps service;
 * see GoogleMapsService for details of the service.
 */
public class GoogleMapsImpl
  implements GoogleMapsService
{
  private static final String API_KEY = "ABQIAAAA7kHuyDenRy7D_kXwDkUfhBQCabR54RQscLxLTjQlrb8wKm07EBRSANMlyMuVIxp6jUQazrN52Pzp3w";
  private static final String API_URL = "http://maps.google.com/maps/geo";

  /**
  * Converts a location string to latitude and longitude coordinates.
  * The response is in the following format:
  *<pre>
  * <?xml version="1.0" encoding="UTF-8"?>
   * <kml xmlns="http://earth.google.com/kml/2.0">
   *   <Response>
   *     <name>Salt Lake City, UT</name>
   *     <Status>
   *       <code>200</code>
   *       <request>geocode</request>
   *     </Status>
   *     <Placemark id="p1">
   *       ...
   *       <Point>
   *         <coordinates>-111.888189,40.771592,0</coordinates>
   *       </Point>
   *     </Placemark>
   *   </Response>
   * </kml>
  * </pre>
  *
   * @param locationStr - the location string to convert
   * @param latLong - the structure to hold the latitude and longitude
   * @throws Exception
   */
  public void locationToLatLong(String locationStr, LatitudeLongitude latLong)
    throws Exception
  {
    String encodedLocationStr = URLEncoder.encode(locationStr, "UTF-8");
    String url = API_URL + "?q=" + encodedLocationStr + "&output=xml&key=" + API_KEY;
    String response = HTTPUtils.sendHTTPRequest(url, null);
    
    System.out.println("Maps.locationToLatLong response [" + response + "]");

    DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
    Document doc = docBuilder.parse(new InputSource(new StringReader(response)));
    if (null != doc)
    {
      NodeList nodeList = doc.getElementsByTagName("coordinates");
      if (null != nodeList && nodeList.getLength() > 0)
      {
        Node locationNode = nodeList.item(0);
        if (null != locationNode)
        {
          String longLatAltStr = locationNode.getTextContent();
          //System.out.println("Maps.locationToLatLong latLongStr [" + longLatAltStr + "]");
          StringTokenizer tokenizer = new StringTokenizer(longLatAltStr, ",");
          if (tokenizer.hasMoreTokens())
          {
            latLong.longitude = tokenizer.nextToken();
            latLong.latitude = tokenizer.nextToken();
            return;
          }
        }
      }
    }

    throw new Exception("Invalid data returned in Maps.locationToLatLong");
  }

  public Object execute(Properties props)
  {
    System.out.println(getClass().getName() + ".execute()");

    Object locationObj = props.get("location");
    LatitudeLongitude latLong = new LatitudeLongitude("", "");
    
    try
    {
      locationToLatLong((String)locationObj, latLong);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
    
    return latLong;
  }
}
