package org.mashups4j.services.registry;

import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceEvent;

import java.util.Properties;

import org.mashups4j.services.twitter.*;
import org.mashups4j.services.google.*;
import org.mashups4j.services.twoogle.*;

/**
 * User: jhanson
 * Date: Jul 13, 2008
 * Time: 2:33:04 PM
 */

/**
 * This class implements a simple bundle that uses the bundle
 * context to register several services
 * with the OSGi framework.
 */
public class ServiceProvider
  implements BundleActivator,
             ServiceListener
{
  /**
   * Implements BundleActivator.start(). Registers a number of services using the bundle context;
   * attaches properties to the services that can be queried
   * when performing a service look-up.
   *
   * @param context the framework context for the bundle.
   */
  public void start(BundleContext context)
  {
    System.out.println(getClass().getName() + ".start()");
    context.addServiceListener(this);
    
    Properties twitterProps = new Properties();
    twitterProps.put("API_PATH", "/twitter");
    context.registerService(TwitterService.class.getName(),
                            new TwitterImpl(), twitterProps);
                            
    Properties googleMapsProps = new Properties();
    googleMapsProps.put("API_PATH", "/googlemaps");
    context.registerService(GoogleMapsService.class.getName(),
                            new GoogleMapsImpl(), googleMapsProps);
                            
    Properties twoogleProps = new Properties();
    twoogleProps.put("API_PATH", "/twoogle");
    context.registerService(TwoogleService.class.getName(),
                            new TwoogleImpl(), twoogleProps);
                            
    System.out.println(getClass().getName() + " started");
  }

  /**
   * Implements BundleActivator.stop(). Does nothing since
   * the framework will automatically unregister any registered services.
   *
   * @param context the framework context for the bundle.
   */
  public void stop(BundleContext context)
  {
    System.out.println(getClass().getName() + ".stop()");
    context.removeServiceListener(this);    
    // NOTE: The service is automatically unregistered.
    System.out.println(getClass().getName() + " stopped");
  }

  public void serviceChanged(ServiceEvent event)
  {
    String[] objectClass = (String[])event.getServiceReference().getProperty("objectClass");

    if (event.getType() == ServiceEvent.REGISTERED)
    {
      System.out.println(getClass().getName() + ": Service of type " + objectClass[0] + " registered.");
    }
    else if (event.getType() == ServiceEvent.UNREGISTERING)
    {
      System.out.println(getClass().getName() + ": Service of type " + objectClass[0] + " unregistered.");
    }
    else if (event.getType() == ServiceEvent.MODIFIED)
    {
      System.out.println(getClass().getName() + ": Service of type " + objectClass[0] + " modified.");
    }
    else
    {
      System.out.println(getClass().getName() + ": Service of unknown type " + event.getType());
    }
  }
}